@@ -1,6 +1,6 @@
 // This defines the API exposed by the Python extension module.
 //
-// Copyright (c) 2016 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2018 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of PyQt5.
 // 
@@ -33,6 +33,7 @@ void fetchtr_ui(const char *fileName, MetaTranslator *tor,
         const char *defaultContext, bool mustExist);
 
 void merge(const MetaTranslator *tor, const MetaTranslator *virginTor,
-        MetaTranslator *out, bool verbose, bool noObsolete);
+        MetaTranslator *out, bool noObsolete, bool verbose,
+        const QString &filename);
 
 #endif
